@@ -56,13 +56,14 @@ extern int show_synth_soa_serial();
  *   "\007example\003com\000\000\001" We will call this SBlabel
  */
 
-/* Function to initialize the csv2_add_state for a new zone
+/* Function to initialize the csv2_add_state for a new zone; this would
+ * be in the constructor if we were coding in C++ 
  * Input: The zone in Alabel format.
  * Output: The initialized csv2 add state
  */
 
 csv2_add_state *csv2_init_add_state(js_string *zone) {
-	js_string *nzone;
+	js_string *nzone, *norigin;
 	csv2_add_state *new;
 	if(zone->unit_size != 1) {
 		return 0;
@@ -74,22 +75,83 @@ csv2_add_state *csv2_init_add_state(js_string *zone) {
 		js_dealloc(new);
 		return 0;
 	}
+	if((norigin = js_create(zone->unit_count + 1,1)) == 0) {
+		js_dealloc(new);
+		return 0;
+	}
 	if(js_copy(zone,nzone) == JS_ERROR) {
 		js_destroy(nzone);
 		js_dealloc(new);
 		return 0;
 	}
+	if(js_copy(zone,norigin) == JS_ERROR) {
+		js_destroy(nzone);
+		js_dealloc(new);
+		return 0;
+	}
 	new->buffer = 0;
-	new->zone = nzone;
+	new->zone = nzone; /* This is the "origin" */
+	new->origin = norigin; /* This is the "origin" */
+	new->ostack = 0; /* A pointer to a "stack" or origin values we can
+                          * push or pop */
+	new->ostack_height = 0;
 	new->rrnum = 0;
 	new->bighash = 0;
 	new->add_method = 0;
 	new->soa_serial = 1;
+	new->default_ttl = 86400;
 	new->in_ns = 1;
 	new->zone_nses_added = 0;
 	return new;
 }
 
+void csv2_zap_add_state(csv2_add_state *x) {
+	int a = 0;
+	csv2_origin *v, *q;
+	csv2_rr *y, *z;
+
+	/* Get rid of the origin stack */
+	v = x->ostack;
+	while(v != 0 && a < 10000) {
+		js_destroy(v->origin);
+		q = v->next;
+		js_dealloc(v);
+		v = q;
+		a++;
+	}
+	a = 0;
+
+	/* Get rid of the buffer of rrs */
+	y = x->buffer;
+	while(y != 0 && a < 10000) {
+		js_destroy(y->query);
+		js_destroy(y->data);
+		z = y->next;
+		js_dealloc(y);
+		y = z;
+		a++;
+	}
+	a = 0;
+
+	/* Get rid of the zone string */
+	if(x->zone != 0) {
+		js_destroy(x->zone);
+		x->zone = 0;
+	}
+	/* ...the origin string */
+	if(x->origin != 0) {
+		js_destroy(x->origin);
+		x->origin = 0;
+	}
+
+	/* And now, finally, get rid of this object, I mean structure */
+	js_dealloc(x);
+}
+	
+/* Function to clean up the memory used by a given csv2_add_state; if this
+ * were object-oriented, this function would be the deconstructor for the
+ * csv2_add_state */
+
 /* Function to set the big hash we point to 
  * Input: Pointer to hash
  * Output: JS_ERROR on error; JS_SUCCESS on success */
@@ -115,7 +177,8 @@ int csv2_set_add_method(csv2_add_state *state, int method) {
 /* Function that closes out the state for processing parsed records;
  * JS_ERROR on error, JS_SUCCESS on success */
 int csv2_close_state(csv2_add_state *state) {
-	return js_dealloc(state);
+	csv2_zap_add_state(state);
+	return JS_SUCCESS;
 }
 
 /* Function that uses stat() to determine the timestamp to put in the
@@ -230,6 +293,9 @@ int csv2_add_rr_bighash(csv2_add_state *state, js_string *query,
 	if(state->zone == 0) {
 		return JS_ERROR;
 	}
+	if(state->origin == 0) {
+		return JS_ERROR;
+	}
 	if(js_has_sanity(query) == JS_ERROR) {
 		return JS_ERROR;
 	}
@@ -846,6 +912,10 @@ int csv2_pop_buffer(csv2_add_state *state) {
 		ret = JS_SUCCESS; 
 	}
 	if(ret != JS_ERROR) {
+		js_destroy(state->buffer->query); /* This is copied in
+                                                     csv2_add_rr_w */
+		js_destroy(state->buffer->data); /* We finally copy this in 
+                                                  * mhash_put_rr/mhash_add_rr */
 		js_dealloc(state->buffer);
 		state->buffer = save;
 	}
