@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2005 Sam Trenholme
+/* Copyright (c) 2002-2006 Sam Trenholme
  *
  * TERMS
  *
@@ -285,7 +285,8 @@ int make_ip_acl(js_string *in, ipv4pair *out, int max, int depth) {
         return JS_ERROR;
         }
     if(js_str2js(delimq,&delim,1,1) == JS_ERROR) {
-        js_destroy(sub); js_destroy(delimq); return JS_ERROR;
+        js_destroy(sub); js_destroy(value); js_destroy(delimq); 
+        return JS_ERROR;
         }
 
     next = -1;
@@ -294,7 +295,7 @@ int make_ip_acl(js_string *in, ipv4pair *out, int max, int depth) {
 	/* Look for the next comma */
         next = js_match_offset(delimq,in,start);
         if(next == JS_ERROR) {
-            js_destroy(sub); js_destroy(delimq);
+            js_destroy(sub); js_destroy(delimq); js_destroy(value);
             return JS_ERROR;
             }
         /* If there is no next comma */
@@ -311,13 +312,13 @@ int make_ip_acl(js_string *in, ipv4pair *out, int max, int depth) {
                     nx--;
 		    }
 		else {
-                    js_destroy(sub); js_destroy(delimq);
+                    js_destroy(sub); js_destroy(delimq); js_destroy(value);
                     return JS_ERROR;
                     }
                 }
 	    /* Make the string "sub" go to the end of the string */
             if(js_substr(in,sub,start,(nx + 1) - start) == JS_ERROR) {
-                js_destroy(sub); js_destroy(delimq);
+                js_destroy(sub); js_destroy(delimq); js_destroy(value);
                 return JS_ERROR;
                 }
             }
@@ -336,25 +337,25 @@ int make_ip_acl(js_string *in, ipv4pair *out, int max, int depth) {
                     nx--;
 		    }
 		else {
-                    js_destroy(sub); js_destroy(delimq);
+                    js_destroy(sub); js_destroy(delimq); js_destroy(value);
                     return JS_ERROR;
                     }
                 }
 	    /* Make the string "sub" go to the next comma */
             if(js_substr(in,sub,start,(nx + 1) - start) == JS_ERROR) {
-                js_destroy(sub); js_destroy(delimq);
+                js_destroy(sub); js_destroy(delimq); js_destroy(value);
                 return JS_ERROR;
                 }
             }
         /* See if an alias for the value exists */
         if(js_qstr2js(value,"ipv4_alias") == JS_ERROR) {
-            js_destroy(sub); js_destroy(delimq);
+            js_destroy(sub); js_destroy(delimq); js_destroy(value);
             return JS_ERROR;
             }
         /* If so, recurse */
         if(read_dvar(value,sub,value) != JS_ERROR) {
             if(make_ip_acl(value,out,max,depth + 1) == JS_ERROR) {
-                js_destroy(sub); js_destroy(delimq);
+                js_destroy(sub); js_destroy(delimq); js_destroy(value);
                 return JS_ERROR;
                 }
             }
@@ -365,11 +366,12 @@ int make_ip_acl(js_string *in, ipv4pair *out, int max, int depth) {
                 index++;
             /* Error if we are out of bounds */
             if(index >= max) {
-                js_destroy(sub); js_destroy(delimq); return JS_ERROR;
+                js_destroy(sub); js_destroy(delimq); js_destroy(value);
+		return JS_ERROR;
                 }
             /* Run ddip_ip_mask to convert the actual ip/mask pair */
             if(ddip_ip_mask(sub,&(out[index]),0) == JS_ERROR) {
-                js_destroy(sub); js_destroy(delimq);
+                js_destroy(sub); js_destroy(delimq); js_destroy(value);
                 return JS_ERROR;
                 }
             }
@@ -377,6 +379,7 @@ int make_ip_acl(js_string *in, ipv4pair *out, int max, int depth) {
 
     js_destroy(sub);
     js_destroy(delimq);
+    js_destroy(value);
     return JS_SUCCESS;
     }
 
